
{

  open Lexing
  open Lp_parser

  let ident (loc,id) =
    if id="is" then IS(loc) else ID(loc,id)
}

let letter = ['a'-'z' 'A'-'Z' '0'-'9' '\'' '_']
let infix = ['+' '-' '*' '/' '\\' '=' '<' '>']

rule token = parse
  | [' ' '\r' '\t'] 
      {  token lexbuf }
  | '\n' | '%' [^'\n']* '\n'
      { let ln = lexbuf.lex_curr_p.pos_lnum
	and off = lexbuf.lex_curr_p.pos_cnum
	in
	lexbuf.lex_curr_p <- { lexbuf.lex_curr_p with
				 pos_lnum = ln+1; pos_bol = off};
	token lexbuf }
  | "/*"
      { comment lexbuf }
  | ":-"                
      { COLONMINUS }
  | "("
      { LPAR }
  | ")"
      { RPAR }
  | ","
      { COMMA }
  | "."
      { DOT }
  | '['
      { LSQBRACKET }
  | ']'
      { RSQBRACKET }
  | '|'
      { BAR }
  | ['A'-'Z'] letter* | '_'
      { VAR (Lexing.lexeme_start_p lexbuf, Lexing.lexeme lexbuf) }
  | ['a'-'z' '0'-'9'] letter*
      { ident (Lexing.lexeme_start_p lexbuf, Lexing.lexeme lexbuf) }
  | infix | "=<" | "=\\=" 
      { INFIX (Lexing.lexeme_start_p lexbuf, Lexing.lexeme lexbuf) }
  | _ 
      { failwith ("Unrecognized character " ^ (Lexing.lexeme lexbuf)) }
  | eof 
      { EOF }

and comment = parse
  | "*/" 
      { token lexbuf }
  | '\n' 
      { let ln = lexbuf.lex_curr_p.pos_lnum
	and off = lexbuf.lex_curr_p.pos_cnum
	in
	lexbuf.lex_curr_p <- { lexbuf.lex_curr_p with
				 pos_lnum = ln+1; pos_bol = off};
	comment lexbuf }
  | [^ '\n']
      { comment lexbuf }
  | eof
      { failwith "Unterminated comment" }



