%{

  open Lp_ast

%}

%token EOF
%token LPAR RPAR COMMA COLONMINUS DOT
%token RSQBRACKET LSQBRACKET BAR
%token <Lexing.position * string> ID VAR INFIX
%token <Lexing.position> IS

%type < Lp_ast.decl list > spec
%start spec
%%

spec:
| decl spec   { $1::$2 }
| EOF         { [] }
;

decl:
| lit DOT                    { Clause($1,[]) }
| lit COLONMINUS litlist DOT { Clause($1,$3) }
| COLONMINUS lit DOT         { Query($2) }
;

lit:
| ident LPAR termlist RPAR  { Pred($1,$3) }
| term INFIX term       { Pred($2,[$1;$3]) }
| term IS term       { Pred(($2,"is"),[$1;$3]) }
;

litlist:
| lit                 { [$1] }
| lit COMMA litlist   { $1::$3 }
;

termlist:
| term                { [$1] }
| term COMMA termlist { $1::$3 }
;

term:
| listnotation          { $1 }
| VAR                   { Var($1) }
| ident                    { Term($1,[]) }
| ident LPAR termlist RPAR { Term($1,$3) }
| term INFIX term       { Term($2,[$1;$3]) }
| LPAR termlist RPAR    { Term((Lexing.dummy_pos,"tuple"),$2) }
;

listnotation:
| LSQBRACKET RSQBRACKET { Term((Lexing.dummy_pos,"[]"),[]) }
| LSQBRACKET conslist RSQBRACKET { $2 }
;

conslist:
| term                { Term((Lexing.dummy_pos,"|"),[$1]) }
| term COMMA conslist { Term((Lexing.dummy_pos,"|"),[$1;$3]) }
| term BAR term       { Term((Lexing.dummy_pos,"|"),[$1;$3]) }
;

ident:
| ID { $1 }
| IS { ($1,"is") }
;






